diff -urN ../postfix-2.0.0.2.dist/src/util/Makefile.in src/util/Makefile.in
--- ../postfix-2.0.0.2.dist/src/util/Makefile.in	Tue Jan 14 15:28:24 2003
+++ src/util/Makefile.in	Tue Jan 14 15:28:24 2003
@@ -6,6 +6,7 @@
 	dict_ht.c dict_ldap.c dict_mysql.c dict_ni.c dict_nis.c \
 	dict_nisplus.c dict_open.c dict_pcre.c dict_regexp.c dict_static.c \
 	dict_tcp.c dict_unix.c dir_forest.c doze.c duplex_pipe.c \
+	dict_whoson.c \
 	environ.c events.c exec_command.c fifo_listen.c fifo_trigger.c \
 	file_limit.c find_inet.c fsspace.c fullname.c get_domainname.c \
 	get_hostname.c hex_quote.c htable.c inet_addr_host.c \
@@ -34,6 +35,7 @@
 	dict_ht.o dict_ldap.o dict_mysql.o dict_ni.o dict_nis.o \
 	dict_nisplus.o dict_open.o dict_pcre.o dict_regexp.o dict_static.o \
 	dict_tcp.o dict_unix.o dir_forest.o doze.o duplex_pipe.o \
+	dict_whoson.o \
 	environ.o events.o exec_command.o fifo_listen.o fifo_trigger.o \
 	file_limit.o find_inet.o fsspace.o fullname.o get_domainname.o \
 	get_hostname.o hex_quote.o htable.o inet_addr_host.o \
@@ -60,6 +62,7 @@
 	dict_ht.h dict_ldap.h dict_mysql.h dict_ni.h dict_nis.h \
 	dict_nisplus.h dict_pcre.h dict_regexp.h dict_static.h dict_tcp.h \
 	dict_unix.h dir_forest.h events.h exec_command.h find_inet.h \
+	dict_whoson.h \
 	fsspace.h fullname.h get_domainname.h get_hostname.h hex_quote.h \
 	htable.h inet_addr_host.h inet_addr_list.h inet_addr_local.h \
 	inet_util.h intv.h iostuff.h line_wrap.h listen.h lstat_as.h \
diff -urN ../postfix-1.1.11.dist/src/util/dict_open.c src/util/dict_open.c
--- ../postfix-1.1.11.dist/src/util/dict_open.c	Tue Sep 10 10:57:15 2002
+++ src/util/dict_open.c	Tue Sep 10 11:00:47 2002
@@ -169,6 +169,7 @@
 #include <dict_pcre.h>
 #include <dict_regexp.h>
 #include <dict_static.h>
+#include <dict_whoson.h>
 #include <stringops.h>
 #include <split_at.h>
 #include <htable.h>
@@ -216,6 +217,7 @@
     DICT_TYPE_REGEXP, dict_regexp_open,
 #endif
     DICT_TYPE_STATIC, dict_static_open,
+    DICT_TYPE_WHOSON, dict_whoson_open,
     0,
 };
 
diff -urN ../postfix-1.1.11.dist/src/util/dict_whoson.c src/util/dict_whoson.c
--- ../postfix-1.1.11.dist/src/util/dict_whoson.c	Thu Jan  1 01:00:00 1970
+++ src/util/dict_whoson.c	Tue Sep 10 10:57:50 2002
@@ -0,0 +1,269 @@
+/*++
+/* NAME
+/*	dict_whoson 3
+/* SUMMARY
+/*	dictionary manager interface to whoson lookup tables
+/* SYNOPSIS
+/*	#include <dict_whoson.h>
+/*
+/*	DICT	*dict_whoson_open(map, dummy, dict_flags)
+/*	const char *map;
+/*	int	dummy;
+/*	int	dict_flags;
+/* DESCRIPTION
+/*      dict_whoson_open() make a WHOSON server accessible
+/*      via the generic dictionary operations described in 
+/*      dict_open(3).
+/*	The \fIdummy\fR argument is not used. The only implemented
+/*	operation is dictionary lookup. 
+/*
+/*      Map names have the form host:port in case of a TCP
+/*      whoson server or /path/to/socket in case of a Unix domain
+/*      server.
+/*
+/*      WHOSON protocol is designed to allow mail relay to
+/*      a given IP adress on a smtpd server without requiring
+/*      an SMTP authentication. Authentication has to be trusted
+/*      from another source (for example, a POP server)
+/*
+/*      Keys have to be full IP addresses. If a value is found
+/*      for a key, lookup reply will be the constant string "OK",
+/*      the IP address key and the whoson server answer are logged.
+/*
+/* SEE ALSO
+/*	dict(3) generic dictionary manager
+/* DIAGNOSTICS
+/*	Fatal errors: out of memory, unknown host or service name,
+/*	attempt to update or iterate over map.
+/*      Warnings: unable to connect to whoson server, connection
+/*      close, whoson server error.
+/* BUGS
+/*	Only the lookup method is currently implemented.
+/* LICENSE
+/* .ad
+/* .fi
+/*	The Secure Mailer license must be distributed with this software.
+/* AUTHOR(S)
+/*	Wietse Venema
+/*	IBM T.J. Watson Research
+/*	P.O. Box 704
+/*	Yorktown Heights, NY 10598, USA
+/*--*/
+
+/* System library. */
+
+#include "sys_defs.h"
+#include <unistd.h>
+#include <string.h>
+
+#include "msg.h"
+#include "mymalloc.h"
+#include "vstring.h"
+#include "vstream.h"
+#include "vstring_vstream.h"
+#include "valid_hostname.h"
+#include "connect.h"
+#include "stringops.h"
+#include "dict_whoson.h"
+#include "dict.h"
+
+
+typedef struct {
+  DICT    dict;                       /* generic members */
+  VSTRING *input;                     /* raw I/O buffer */
+  VSTRING *dummy;                     /* raw I/O buffer to skip empty lines */
+  VSTREAM *fp;                        /* I/O stream */
+} DICT_WHOSON;
+
+#define DICT_WHOSON_TMOUT  100
+
+/*
+ * Does the server stop when the whoson server fails ?
+ *
+ * According the nature of the map, it should be a bad thing
+ * exepted on a dedicaced SMTP after POP server.
+ */
+
+#undef DICT_WHOSON_INSISTS
+
+#ifdef DICT_WHOSON_INSISTS
+#  define DICT_WHOSON_MAXTRY 10
+#  define DICT_WHOSON_ERROR  DICT_ERR_RETRY
+#else
+#  define DICT_WHOSON_MAXTRY 10
+#  define DICT_WHOSON_ERROR  DICT_ERR_NONE
+#endif
+
+
+#define STR(x)		vstring_str(x)
+
+/* dict_whoson_connect - connect to whoson server */
+
+static int dict_whoson_connect(DICT_WHOSON *dict_whoson)
+{
+  int fd;
+  if (dict_whoson->dict.name[0] == '/') { /* unix domain socket */
+    fd = unix_connect(dict_whoson->dict.name, BLOCKING, 0);
+  } else {
+    fd = inet_connect(dict_whoson->dict.name, BLOCKING, 0);
+  }
+  if (fd < 0) {
+    msg_warn("connect to WHOSON map %s: %m", dict_whoson->dict.name);
+    return -1;
+  }
+
+  dict_whoson->fp = vstream_fdopen(fd, O_RDWR);
+
+  vstream_control(dict_whoson->fp,
+		  VSTREAM_CTL_TIMEOUT, DICT_WHOSON_TMOUT,
+		  VSTREAM_CTL_END);
+
+  dict_whoson->fp = vstream_fdopen(fd, O_RDWR);
+
+  if (dict_whoson->input == 0) {
+    dict_whoson->input = vstring_alloc(10);
+    dict_whoson->dummy = vstring_alloc(10);
+  }
+  return 0;
+  
+}
+
+/* dict_whoson_disconnect - disconnect from whoson server */
+
+static void dict_whoson_disconnect(DICT_WHOSON *dict_whoson)
+{
+  (void) vstream_fclose(dict_whoson->fp);
+  dict_whoson->fp = 0;
+}
+
+/* dict_whoson_lookup - request whoson server */
+
+
+static const char *dict_whoson_lookup(DICT *dict, const char *key)
+{
+  DICT_WHOSON *dict_whoson = (DICT_WHOSON *) dict;
+  char   *myname = "dict_whoson_lookup";
+  char   *reply;
+  char   *end_of_reply;
+  char   *dummy;
+  int    tries;
+  
+#define RETURN(errval, result) { dict_errno = errval; return (result); }
+
+  if (msg_verbose)
+    msg_info("%s: key %s", myname, key);
+
+  if (valid_hostaddr(key, 0) == 0) {
+    if (msg_verbose)
+      msg_info("%s: %s is not a host address, dont send it to whoson server",
+	       myname, key);
+    RETURN(DICT_ERR_NONE, 0);
+  }
+
+  for (tries = 0; /* see below */ ; /* see below */ ) {
+
+    /*
+     * Connect to the server, or use an existing connection.
+     */
+    if (dict_whoson->fp != 0 || dict_whoson_connect(dict_whoson) == 0) {
+
+      vstream_fprintf(dict_whoson->fp, "QUERY %s\r\n\r\n", key);
+      if (vstring_get_nonl(dict_whoson->input, dict_whoson->fp) > 0) {
+	
+	reply = STR(dict_whoson->input);
+
+	/*
+	 *  wait for an empty line
+	 */
+	while(vstring_get_nonl(dict_whoson->dummy, dict_whoson->fp)>0) {
+	  dummy = STR(dict_whoson->dummy);
+	  if (dummy[0] == '\r')  dummy ++;
+	  if (dummy[0] == '\0')  break;
+	}
+
+	end_of_reply=reply + strlen(reply);
+	if (reply != end_of_reply && end_of_reply[-1] == '\r') 
+	  end_of_reply[-1]='\0';
+
+	/*
+	 *  Parse the answer
+	 */ 
+
+	if (reply[0] == '+') {
+	  /*
+	   *  send the anwser in log file
+	   */
+	  msg_info("%s: user found '%.100s' for [%s] from %s", myname,
+		   reply+1, key, dict_whoson->dict.name);
+
+	  RETURN(DICT_ERR_NONE, "OK");
+
+	} else if (reply[0] == '-') {
+	  if (msg_verbose) {
+	    msg_info("%s: user not found for [%s] from %s", myname,
+		     key, dict_whoson->dict.name);
+	  }
+	  RETURN(DICT_ERR_NONE, 0);
+
+	} else if (reply[0] == '*') {
+	  msg_warn("%s: read whoson map reply from %s: %.100s", myname,
+		   dict_whoson->dict.name, printable(reply, '?'));
+	  RETURN(DICT_WHOSON_ERROR, 0);
+	} else {
+	  msg_warn("%s: read whoson map reply from %s: bad sequence",
+		   myname, dict_whoson->dict.name);
+	  dict_whoson_disconnect(dict_whoson);
+	  RETURN(DICT_WHOSON_ERROR, 0);
+	}
+      } else {
+	msg_warn("%s: read whoson map reply from %s: unexpected EOF (%m)",
+		 myname, dict_whoson->dict.name);
+	dict_whoson_disconnect(dict_whoson);
+      }
+
+    }
+
+      /*
+       * Try to connect a limited number of times before giving up.
+       */
+      if (++tries >= DICT_WHOSON_MAXTRY)
+	RETURN(DICT_WHOSON_ERROR, 0);
+      
+      /*
+       * Sleep between attempts, instead of hammering the server.
+       */
+      sleep(1);
+
+  }
+  
+}
+
+/* dict_whoson_close - close whoson MAP */
+
+static void dict_whoson_close(DICT *dict) {
+  DICT_WHOSON *dict_whoson = (DICT_WHOSON *) dict;
+
+  if (dict_whoson->fp)
+    (void) vstream_fclose(dict_whoson->fp);
+  if (dict_whoson->input)
+    vstring_free(dict_whoson->input);
+  if (dict_whoson->dummy)
+    vstring_free(dict_whoson->dummy);
+  dict_free(dict);
+}
+
+/* dict_whoson_open - open whoson MAP */
+
+DICT    *dict_whoson_open(const char *map, int unused_flags, int dict_flags)
+{
+  DICT_WHOSON *dict_whoson;
+  dict_errno = 0;
+  dict_whoson = (DICT_WHOSON *) dict_alloc(DICT_TYPE_WHOSON, map,
+					sizeof(*dict_whoson));
+  dict_whoson->dict.lookup = dict_whoson_lookup;
+  dict_whoson->input = dict_whoson->dummy = 0;
+  dict_whoson->dict.flags = dict_flags & ~DICT_FLAG_FIXED;
+  dict_whoson->dict.close = dict_whoson_close;
+  dict_whoson->fp = 0;
+  return (DICT_DEBUG(&dict_whoson->dict));
+}
diff -urN ../postfix-1.1.11.dist/src/util/dict_whoson.h src/util/dict_whoson.h
--- ../postfix-1.1.11.dist/src/util/dict_whoson.h	Thu Jan  1 01:00:00 1970
+++ src/util/dict_whoson.h	Tue Sep 10 10:57:50 2002
@@ -0,0 +1,37 @@
+#ifndef _DICT_WHOSON_H_INCLUDED_
+#define _DICT_WHOSON_H_INCLUDED_
+
+/*++
+/* NAME
+/*	dict_whoson 3h
+/* SUMMARY
+/*	dictionary manager interface to whoson-based lookup tables
+/* SYNOPSIS
+/*	#include <dict_whoson.h>
+/* DESCRIPTION
+/* .nf
+
+ /*
+  * Utility library.
+  */
+#include <dict.h>
+
+ /*
+  * External interface.
+  */
+#define DICT_TYPE_WHOSON	"whoson"
+
+extern DICT *dict_whoson_open(const char *, int, int);
+
+/* LICENSE
+/* .ad
+/* .fi
+/*	The Secure Mailer license must be distributed with this software.
+/* AUTHOR(S)
+/*	Wietse Venema
+/*	IBM T.J. Watson Research
+/*	P.O. Box 704
+/*	Yorktown Heights, NY 10598, USA
+/*--*/
+
+#endif
